/****************************************************************************
**
** Copyright (C) 2001-2006 Tilo Riemer <riemer@crossvc.com>,
**                         Frank Hemer <frank@hemer.org>,
**                         Jose Hernandez <joseh@tesco.net>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** CrossVC is available under two different licenses:
**
** If CrossVC is linked against the GPLed version of Qt 
** CrossVC is released under the terms of GPL also.
**
** If CrossVC is linked against a nonGPLed version of Qt 
** CrossVC is released under the terms of the 
** CrossVC License for non-Unix platforms (CLNU)
**
**
** CrossVC License for non-Unix platforms (CLNU):
**
** Redistribution and use in binary form, without modification, 
** are permitted provided that the following conditions are met:
**
** 1. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 2. It is not permitted to distribute the binary package under a name
**    different than CrossVC.
** 3. The name of the authors may not be used to endorse or promote
**    products derived from this software without specific prior written
**    permission.
** 4. The source code is the creative property of the authors.
**    Extensions and development under the terms of the Gnu Public License
**    are limited to the Unix platform. Any distribution or compilation of 
**    the source code against libraries licensed other than gpl requires 
**    the written permission of the authors.
**
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR 
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
** ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
** WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
**
**
** CrossVC License for Unix platforms:
**
** This program is free software; you can redistribute it and/or modify 
** it under the terms of the GNU General Public License as published by 
** the Free Software Foundation, version 2 of the License.
** This program is distributed in 
** the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
** even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
** PARTICULAR PURPOSE.
**
** See the GNU General Public License version 2 for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
*****************************************************************************/

#include "config.h"

#include <qfileinfo.h>
#include <qlistview.h>
#include <qcombobox.h>
#include <qwhatsthis.h>
#include <qpushbutton.h>
#include <qcheckbox.h>

#include "globals.h"
#include "AnnotateGrepLinesDialogImpl.h"

AnnotateGrepLinesDialogImpl::AnnotateGrepLinesDialogImpl(
							 const QIconSet &whatsThisIconSet,
							 QWidget* parent,
							 const char* name, bool modal, WFlags fl)
//   : AnnotateGrepLinesDialog(LookAndFeel::g_b0AsParent ? 0 : parent, name, modal, fl)
  : AnnotateGrepLinesDialog(modal ? parent : (LookAndFeel::g_b0AsParent ? 0 : parent),
			    name, modal, fl)
{
   m_pWhatsThis->setIconSet(whatsThisIconSet);
#ifdef Q_WS_MAC
  m_pWhatsThis->setMaximumWidth(m_pWhatsThis->height() * 2);
#else
  m_pWhatsThis->setMaximumWidth(m_pWhatsThis->height());
#endif
}

//----------------------------------------------------------------------------

AnnotateGrepLinesDialogImpl::~AnnotateGrepLinesDialogImpl()
{
}

//----------------------------------------------------------------------------

QStringList AnnotateGrepLinesDialogImpl::getSearchLines()
{
   QStringList list;
   QListViewItem *myChild = m_SearchLines->firstChild();
   while(myChild) {
      list.append(myChild->text(0));
      myChild = myChild->nextSibling();
   }
   return list;
}

//----------------------------------------------------------------------------

bool AnnotateGrepLinesDialogImpl::isCaseSensitive()
{
   return m_bCaseSensitive->isChecked();
}

//----------------------------------------------------------------------------

void AnnotateGrepLinesDialogImpl::addSearchLine()
{
   if (m_SearchLine->currentText().stripWhiteSpace().length()) {
      new QListViewItem(m_SearchLines, m_SearchLine->currentText());
      m_SearchLine->clear();
   }
}

//----------------------------------------------------------------------------

void AnnotateGrepLinesDialogImpl::removeSearchLine()
{
   QListViewItem* i = m_SearchLines->currentItem();
   if (i) {
      m_SearchLines->takeItem(i);
      delete i;
   }
}

//----------------------------------------------------------------------------

void AnnotateGrepLinesDialogImpl::enterWhatsThisMode()
{
   QWhatsThis::enterWhatsThisMode();
}

//----------------------------------------------------------------------------

