/****************************************************************************
**
** Copyright (C) 2004-2006 Frank Hemer <frank@hemer.org>,
**                         Tilo Riemer <riemer@crossvc.com>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** CrossVC is available under two different licenses:
**
** If CrossVC is linked against the GPLed version of Qt 
** CrossVC is released under the terms of GPL also.
**
** If CrossVC is linked against a nonGPLed version of Qt 
** CrossVC is released under the terms of the 
** CrossVC License for non-Unix platforms (CLNU)
**
**
** CrossVC License for non-Unix platforms (CLNU):
**
** Redistribution and use in binary form, without modification, 
** are permitted provided that the following conditions are met:
**
** 1. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 2. It is not permitted to distribute the binary package under a name
**    different than CrossVC.
** 3. The name of the authors may not be used to endorse or promote
**    products derived from this software without specific prior written
**    permission.
** 4. The source code is the creative property of the authors.
**    Extensions and development under the terms of the Gnu Public License
**    are limited to the Unix platform. Any distribution or compilation of 
**    the source code against libraries licensed other than gpl requires 
**    the written permission of the authors.
**
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR 
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
** ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
** WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
**
**
** CrossVC License for Unix platforms:
**
** This program is free software; you can redistribute it and/or modify 
** it under the terms of the GNU General Public License as published by 
** the Free Software Foundation, version 2 of the License.
** This program is distributed in 
** the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
** even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
** PARTICULAR PURPOSE.
**
** See the GNU General Public License version 2 for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
*****************************************************************************/

#include "GuiEventBlocker.h"

#include <qptrqueue.h>
#include <qapplication.h>
#include <qguardedptr.h>

class GuiEvent {
 public:
   GuiEvent(QObject * obj, QEvent * e) : m_obj(obj),m_event(e) {}
   ~GuiEvent() {m_obj = NULL;}
   QGuardedPtr<QObject> obj() {return m_obj;}
   QEvent * event() {return m_event;}
 private:
   QGuardedPtr<QObject> m_obj;
   QEvent * m_event;
};

static QPtrQueue<GuiEvent> eventQueue;

GuiEventBlocker::GuiEventBlocker(QObject * parent /* = 0 */, const char * name /* = 0 */) : QObject(parent,name) {}
GuiEventBlocker::~GuiEventBlocker() {}

void GuiEventBlocker::setBlocking(bool block) {
   m_block = block;
   if (!block) {
      GuiEvent * e;
      while ( (e = eventQueue.dequeue()) ) {
	 if (e->obj()) {//might have been allready destroyed
	    qApp->postEvent(e->obj(),e->event());
	 } else {
	    //qDebug("Obj of event allready destroyed: %d",e->event()->type());
	    delete e->event();
	 }
	 delete e;
      }
   }
}

bool GuiEventBlocker::isBlocked() {
   return m_block;
}

bool GuiEventBlocker::eventFilter( QObject *o, QEvent *e ) {
   bool retval = FALSE;
   if (m_block && strcmp(o->name(),"stop action") ) {
      //qDebug("type: %d, name: %s",e->type(),o->name());
      // special processing for blocked gui event

      retval = TRUE;
      GuiEvent * ge = NULL;
      switch (e->type()) {
	 case QEvent::MouseButtonPress:
	 case QEvent::MouseButtonRelease:
	 case QEvent::MouseButtonDblClick: {
	    QMouseEvent * me = static_cast<QMouseEvent*>(e);
	    if (me) ge = new GuiEvent(o,new QMouseEvent(*me));
	    break;
	 }
	 case QEvent::ContextMenu: {
	    QContextMenuEvent * me = static_cast<QContextMenuEvent*>(e);
	    if (me) ge = new GuiEvent(o,new QContextMenuEvent(*me)) ;
	    break;
	 }
	 case QEvent::Accel:
	 case QEvent::AccelOverride:
	 case QEvent::KeyPress:
	 case QEvent::KeyRelease: {
	    QKeyEvent * me = static_cast<QKeyEvent*>(e);
	    if (me) ge = new GuiEvent(o,new QKeyEvent(*me));
	    break;
	 }
	 case QEvent::Wheel: {
	    QWheelEvent * me = static_cast<QWheelEvent*>(e);
	    if (me) ge = new GuiEvent(o,new QWheelEvent(*me));
	    break;
	 }
	 case QEvent::Clipboard: {
	    QCustomEvent * me = static_cast<QCustomEvent*>(e);
	    if (me) ge = new GuiEvent(o,new QCustomEvent(*me));
	    break;
	 }
	 case QEvent::MouseMove:
	 case QEvent::DragEnter:
	 case QEvent::DragMove:
	 case QEvent::DragLeave:
	 case QEvent::Drop:
	 case QEvent::DragResponse: {
	    break;
	 }
	 case QEvent::Close: {
	    break;
	 }
	 default: {// standard event processing
	    retval = FALSE;
	    break;
	 }
      }
      if (ge) {
	 //qDebug( "enqueue %d on %s", ge->event()->type(), ge->obj()->name() );
	 eventQueue.enqueue(ge);
      }
   }

   return retval;
}

bool GuiEventBlocker::m_block = FALSE;

