/****************************************************************************
**
** Copyright (C) 2001-2006 Frank Hemer <frank@hemer.org>,
**                         Tilo Riemer <riemer@crossvc.com>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** CrossVC is available under two different licenses:
**
** If CrossVC is linked against the GPLed version of Qt 
** CrossVC is released under the terms of GPL also.
**
** If CrossVC is linked against a nonGPLed version of Qt 
** CrossVC is released under the terms of the 
** CrossVC License for non-Unix platforms (CLNU)
**
**
** CrossVC License for non-Unix platforms (CLNU):
**
** Redistribution and use in binary form, without modification, 
** are permitted provided that the following conditions are met:
**
** 1. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 2. It is not permitted to distribute the binary package under a name
**    different than CrossVC.
** 3. The name of the authors may not be used to endorse or promote
**    products derived from this software without specific prior written
**    permission.
** 4. The source code is the creative property of the authors.
**    Extensions and development under the terms of the Gnu Public License
**    are limited to the Unix platform. Any distribution or compilation of 
**    the source code against libraries licensed other than gpl requires 
**    the written permission of the authors.
**
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR 
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
** ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
** WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
**
**
** CrossVC License for Unix platforms:
**
** This program is free software; you can redistribute it and/or modify 
** it under the terms of the GNU General Public License as published by 
** the Free Software Foundation, version 2 of the License.
** This program is distributed in 
** the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
** even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
** PARTICULAR PURPOSE.
**
** See the GNU General Public License version 2 for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
*****************************************************************************/

#ifndef MENUITEM_H
#define MENUITEM_H

#include "config.h"


#include <qptrvector.h>
#include <qiconset.h>
#include <qpopupmenu.h>

class MenuItem {
 public:
  MenuItem( int menu, const QIconSet &icon, const QString text, const char *member, int accel = 0, int id = -1, int count = 1, int index = -1);

  MenuItem( int menu, const QString text, const char *member, int accel = 0, int id = -1, int count = 1, int index = -1);

  MenuItem( int menu, const QString text, int count = 1);

  ~MenuItem();

  int insertInto(QPopupMenu *,const QObject *) const;

  QIconSet icon()const {return m_icon;}
  QString caption()const {return m_text;}
  const char* slot()const {return m_member;}
  int accel()const {return m_accel;}
  int id()const {return m_id;}
  int index()const {return m_index;}
  int count()const {return m_count;}
  int menu()const {return m_menu;}
  
  void setEnabled(bool enabled) {m_enabled = enabled;}
  bool getEnabled() {return m_enabled;}

  MenuItem* getParent()const {return m_parent;}
  void setParent(MenuItem *);
  void addChild(MenuItem *);
  MenuItem * getFirstChild();
  MenuItem * getNextChild();
  MenuItem* nextItem()const {return getInstance(m_instanceNum+1);}
  MenuItem* nextSortedItem()const;

  int getInstanceNum()const {return m_instanceNum;}

  static MenuItem* getInstance( int i_Num);

  static void setAccel( const MenuItem *,int);
  static MenuItem* first() {return getInstance(0);}
  static MenuItem* last() {return getInstance(m_instance-1);}
  static MenuItem* firstSorted() {return instP->at(0);}
  static MenuItem* lastSorted() {return instP->at(m_instance-1);}

 private:

  static unsigned int m_instance;
  static QPtrVector<MenuItem> * instP;

  void init();

  QPtrVector<MenuItem> * m_childs;
  const QIconSet m_icon;
  const QString m_text;
  const char *m_member;
  int m_accel;
  int m_id;
  int m_count;
  int m_index;
  unsigned int m_instanceNum;
  int m_menu;
  bool m_enabled;
  MenuItem * m_parent;
  bool m_isAutoAdd;
  unsigned int m_childP;
};

#endif

